/* version.h	Define the current version number
 *
 * $Id: version.h,v 1.8 1995/03/19 17:21:06 bdale Exp $
 *
 * Copyright 1991, Michael Westerhof, Sun Microsystems, Inc.
 * This software may be freely used, distributed, or modified, providing
 * this header is not removed.
 *
 */

#define VERS "1.1.0"

